#version 330 core
layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 texCoords;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;


uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

uniform mat4 shadow;

uniform vec3 light_pos;
uniform vec3 cam_pos;

out vec2 TexCoords;
out vec4 shadowCoord;

out VS_OUT{
	vec3 FragPos;
	vec2 TexCoords;
	vec3 TangentLightPos;
	vec3 TangentViewPos;
	vec3 TangentFragPos;
} vs_out;


void main()
{
	gl_Position = projection * view * model * vec4(position, 1.0f);
	vs_out.FragPos = vec3(model * vec4(position, 1.0));
	vs_out.TexCoords = texCoords;

	mat3 normalMatrix = transpose(inverse(mat3(model)));
	vec3 T = normalize(normalMatrix * tangent);
	vec3 B = normalize(normalMatrix * bitangent);
	vec3 N = normalize(normalMatrix * normal);

	mat3 TBN = transpose(mat3(T, B, N));
	vs_out.TangentLightPos = TBN * light_pos;
	vs_out.TangentViewPos = TBN * cam_pos;
	vs_out.TangentFragPos = TBN * vs_out.FragPos;

	TexCoords = texCoords;
	shadowCoord = shadow*vec4(position, 1.0);
}