#version 330 core

in vec2 TexCoords;
in vec4 shadowCoord;

out vec4 FragColor;

in VS_OUT{
	vec3 FragPos;
	vec2 TexCoords;
	vec3 TangentLightPos;
	vec3 TangentViewPos;
	vec3 TangentFragPos;
} fs_in;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture_normal1;
uniform sampler2D texture_specular1;
uniform sampler2DShadow shadow_tex;

//Material properties
uniform vec3 specular_albedo = vec3(0.7);
uniform float specular_power; // = 128.0;
uniform vec3 ambientMaterial;

//uniform vec3 diffuseMaterial;
uniform vec3 specularMaterial;

void main()
{
	// Obtain normal from normal map in range [0,1]
	vec3 normal = texture(texture_normal1, fs_in.TexCoords).rgb;
	// Transform normal vector to range [-1,1]
	normal = normalize(normal * 2.0 - 1.0);  // this normal is in tangent space

	// Get diffuse color
	vec3 color = texture(texture_diffuse1, fs_in.TexCoords).rgb;
	// Ambient
	vec3 ambient = 0.1 * color;
	// Diffuse
	vec3 lightDir = normalize(fs_in.TangentLightPos - fs_in.TangentFragPos);
	float diff = max(dot(lightDir, normal), 0.0);
	vec3 diffuse = diff * color;
	// Specular
	vec3 viewDir = normalize(fs_in.TangentViewPos - fs_in.TangentFragPos);
	vec3 reflectDir = reflect(-lightDir, normal);
	vec3 halfwayDir = normalize(lightDir + viewDir);
	float spec = pow(max(dot(normal, halfwayDir), 0.0), specular_power);
	vec3 specular = vec3(0.2) * spec;

	FragColor = vec4(ambient + diffuse + specular, 1.0f);

	float vShadowColor = textureProj(shadow_tex, shadowCoord);
	
	if (vShadowColor <= 0.5f){
		vShadowColor = 0.5f;
	}

	FragColor = vShadowColor * FragColor;// vec4(texture(texture_diffuse1, TexCoords));
	//FragColor.a = 1.0f;
	//color = vec4(1.0f, 0.0f, 0.0f, 1.0f);
}