#version 330 core

in vec4 shadowCoord;
in vec4 projectedCoord;
in vec4 projectedCoord2;


out vec4 FragColor;

in VS_OUT{
	vec3 FragPos;
	vec2 TexCoords;
	vec3 TangentLightPos;
	vec3 TangentViewPos;
	vec3 TangentFragPos;
} fs_in;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture_normal1;

uniform sampler2D shadow_tex;
uniform sampler2DShadow shadow_depth_tex;

uniform sampler2DShadow projector_depth_tex;
uniform sampler2D proj_tex;

uniform sampler2DShadow projector_depth_tex2;
uniform sampler2D proj_tex2;

//Material properties
uniform float opacity;
uniform float specular_power;

uniform float biasShadow;





void main()
{
	// Obtain normal from normal map in range [0,1]
	vec3 normal = texture(texture_normal1, fs_in.TexCoords).rgb;
	// Transform normal vector to range [-1,1]
	normal = normalize(normal * 2.0 - 1.0);  // this normal is in tangent space

	// Get diffuse color
	vec3 color = texture(texture_diffuse1, fs_in.TexCoords).rgb;
	// Ambient
	vec3 ambient = 0.1 * color;
	// Diffuse
	vec3 lightDir = normalize(fs_in.TangentLightPos - fs_in.TangentFragPos);
	float diff = max(dot(lightDir, normal), 0.0);
	vec3 diffuse = diff * color;
	// Specular
	vec3 viewDir = normalize(fs_in.TangentViewPos - fs_in.TangentFragPos);
	//vec3 reflectDir = reflect(-lightDir, normal);
	vec3 halfwayDir = normalize(lightDir + viewDir);
	float spec = pow(max(dot(normal, halfwayDir), 0.0), specular_power);
	vec3 specular = vec3(0.2) * spec;





	/*float vShadowColor = textureProj(shadow_tex, shadowCoord);

	if (vShadowColor <= 0.5f){
	vShadowColor = 0.5f;
	}*/





	if (shadowCoord.w>1) {

		//divide the shadow coordinate by homogeneous coordinate

		//get the depth value
		//float depth = uv.z;

		vec3 uv = shadowCoord.xyz / shadowCoord.w;




		float depth = uv.z;

		//read the moments from the shadow map texture
		vec4 moments = texture(shadow_tex, uv.xy);

		//calculate variance from the moments
		float E_x2 = moments.y;
		float Ex_2 = moments.x*moments.x;
		float var = E_x2 - Ex_2;

		//bias the variance
		var = max(var, 0.00002);

		//subtract the fragment depth from the  first moment
		//divide variance by the squared difference value
		//to get the maximum probability of fragment to be in shadow
		float mD = depth - moments.x;
		float mD_2 = mD*mD;
		float p_max = var / (var + mD_2);

		//darken the diffuse component if the current depth is less than or equal
		//to the first moment and the retured value is less than the calculated
		//maximum probability

		//if(biasShadow!=0.0f)
		diffuse *= 1.0f - max(p_max, (depth <= moments.x) ? 1.0 : 0.2);

		//diffuse*=1.0f-p_max;
		//FragColor=vec4(depth);
		//FragColor=vec4(depth);
		//diffuse*=(depth<=moments.x)?1.0:0.2;
		//}


		//FragColor=shadowCoord;
	}

	FragColor = vec4(ambient + diffuse + specular, 1.0f);

	//	FragColor=vec4(1.0);


	vec4 vProjectorColor = vec4(1.0, 1.0, 1.0, 0.0);
	vec3 tex_coords = projectedCoord.xyz / projectedCoord.w;
	float vProjShadow = 1.0;
	if ((tex_coords.x >= 0 && tex_coords.x <= 1) &&
		(tex_coords.y >= 0 && tex_coords.y <= 1))
	{
		float vProjectorShadowColor = textureProj(projector_depth_tex, projectedCoord);


		if (vProjectorShadowColor>0.5){
			vProjectorColor = textureProj(proj_tex, tex_coords);
			vProjectorColor.a = 0.5;
			//vProjectorColor=vec4(1.0,0.0, 0.0, 1.0);
			FragColor = FragColor + vProjectorColor;
		}



	}


	vProjectorColor = vec4(1.0, 1.0, 1.0, 0.0);
	tex_coords = projectedCoord2.xyz / projectedCoord2.w;
	vProjShadow = 1.0;
	if ((tex_coords.x >= 0 && tex_coords.x <= 1) &&
		(tex_coords.y >= 0 && tex_coords.y <= 1))
	{
		float vProjectorShadowColor = textureProj(projector_depth_tex2, projectedCoord2);


		if (vProjectorShadowColor>0.5){
			vProjectorColor = textureProj(proj_tex2, tex_coords);
			vProjectorColor.a = 0.5;
			//vProjectorColor=vec4(1.0,0.0, 0.0, 1.0);
			FragColor = FragColor + vProjectorColor;
		}



	}




	//FragColor = vShadowColor * FragColor * vProjectorColor;// vec4(texture(texture_diffuse1, TexCoords));


	FragColor.a = opacity;
	//color = vec4(1.0f, 0.0f, 0.0f, 1.0f);
}